@@ -1670,20 +1670,33 @@ class SQLObject(object):
         return NotImplemented
 
 
+    # (De)serialization (pickle, etc.)
+
     def __getstate__(self):
         if self.sqlmeta._perConnection:
             from pickle import PicklingError
             raise PicklingError('Cannot pickle an SQLObject instance that has a per-instance connection')
+        if self.sqlmeta.lazyUpdate and self._SO_createValues:
+            self.syncUpdate()
         d = self.__dict__.copy()
         del d['sqlmeta']
+        del d['_SO_validatorState']
         del d['_SO_writeLock']
+        del d['_SO_createValues']
         return d
 
     def __setstate__(self, d):
         self.__init__(_SO_fetch_no_create=1)
+        self._SO_validatorState = sqlbuilder.SQLObjectState(self)
         self._SO_writeLock = threading.Lock()
+        self._SO_createValues = {}
         self.__dict__.update(d)
-        self.__class__._connection.cache.put(self.id, self.__class__, self)
+        cls = self.__class__
+        cache = self._connection.cache
+        if cache.tryGet(self.id, cls) is not None:
+            raise ValueError(
+                "Cannot unpickle %s row with id=%s - a different instance with the id already exists in the cache" % (cls.__name__, self.id))
+        cache.created(self.id, cls, self)
 
 
 def setterName(name):
